@@ -18,6 +18,24 @@ import sys
 _coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
 
 
+def frames_after_module(frames, module):
+    started = False
+    last_mod = ''
+    results = []
+    for frame, lineno in list(frames)[::-1]:
+        if not started:
+            f_globals = getattr(frame, 'f_globals', {})
+            module_name = _getitem_from_frame(f_globals, '__name__')
+            if (last_mod and last_mod.startswith('unittest2')) \
+                and not module_name.startswith('unittest2'):
+                started = True
+            else:
+                last_mod = module_name
+                continue
+        results.append((frame, lineno))
+    return results
+
+
 def get_lines_from_file(filename, lineno, context_lines, loader=None, module_name=None):
     """
     Returns context_lines before and after lineno from file.
@@ -45,6 +63,7 @@ def get_lines_from_file(filename, lineno, context_lines, loader=None, module_nam
             source = None
         if source is not None:
             source = source.splitlines()
+
     if source is None:
         try:
             f = open(filename)
@@ -54,6 +73,7 @@ def get_lines_from_file(filename, lineno, context_lines, loader=None, module_nam
                 f.close()
         except (OSError, IOError):
             pass
+
     if source is None:
         return []
 
@@ -68,10 +88,10 @@ def get_lines_from_file(filename, lineno, context_lines, loader=None, module_nam
     source = [unicode(sline, encoding, 'replace') for sline in source]
 
     lower_bound = max(0, lineno - context_lines)
-    upper_bound = lineno + context_lines + 1
+    upper_bound = min(lineno + context_lines + 1, len(source))
 
     try:
-        return [(lineno, source[lineno].strip('\n')) for lineno in xrange(lower_bound, upper_bound)]
+        return [(lineno + 1, source[lineno].strip('\n')) for lineno in xrange(lower_bound, upper_bound)]
     except IndexError:
         # the file may have changed since it was loaded into memory
         return []
